from django.contrib.auth.signals import user_logged_in
from django.dispatch import receiver

from .rules import process_rules


@receiver(user_logged_in)
def logged_expiry_handler(sender, user, request, **kwargs):
    """Processes rules for logged in user."""
    process_rules(request=request, user=user)



from django.conf import settings
from django.utils.module_loading import import_string


def get_settings_key(user):
    return 'AUTH' if user.is_authenticated else 'ANON'


def process_rule(rule, **kwargs):
    """
    Processes a rule. Returns `True` if the rule is valid.

    When the rule is not a callable, tries importing it, assuming it is
    a function defined in a specific module of the application.

    Anonymous rules don't contain an user, so we extract it when calling the
    rule for validation.
    """
    if not callable(rule):
        rule = import_string(rule)

    request = kwargs.pop('request')
    user = kwargs.pop('user', None)
    return rule(request, user) if user.is_authenticated else rule(request)


def process_rules(**kwargs):
    """
    Processes all rules. If a default age is defined, sets the session
    expiry to the default age value.

    Rules will always override the default age.
    """
    request = kwargs.get('request')
    key = get_settings_key(kwargs['user'])

    default_age = getattr(settings, 'EXPIRY_{}_SESSION_AGE'.format(key), None)
    if default_age:
        request.session.set_expiry(default_age)

    rules = getattr(settings, 'EXPIRY_{}_SESSION_RULES'.format(key), ())
    for rule, expiry in rules:
        if process_rule(rule, **kwargs):
            request.session.set_expiry(expiry)
            break



import os

from setuptools import (
    find_packages,
    setup,
)


def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read()


setup(
    name='django-expiry',
    version='0.2.3',
    license='MIT',
    description='Expiry rules for Django sessions.',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    author='Ramon Saraiva',
    author_email='ramonsaraiva@gmail.com',
    url='https://github.com/ramonsaraiva/django-expiry',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    install_requires=[],
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



from django.conf import settings

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    class MiddlewareMixin:
        pass

from .rules import (
    get_settings_key,
    process_rules,
)


class ExpirySessionMiddleware(MiddlewareMixin):

    def process_response(self, request, response):
        """
        If the current session is fresh (was just created by the default
        session middleware, setting its expiry to `SESSION_COOKIE_AGE`)
        or the session is configured to keep alive, processes all rules
        to identify what `expiry` should be set.
        """
        if not (hasattr(request, 'user') and hasattr(request, 'session')):
            return response

        key = get_settings_key(request.user)

        fresh_session = (
            request.session.get_expiry_age() == settings.SESSION_COOKIE_AGE
        )
        keep_alive = getattr(
            settings, 'EXPIRY_{}_KEEP_ALIVE'.format(key), False
        )

        if fresh_session or keep_alive:
            process_rules(request=request, user=request.user)

        return response
